@@ -1,5 +1,7 @@
 """bareASGI-graphql-next"""
 
+import logging
+
 from .graphql.controller import GraphQLController
 from .graphql.helpers import add_graphql_next
 
@@ -7,3 +9,5 @@ __all__ = [
     'GraphQLController',
     'add_graphql_next'
 ]
+
+logging.getLogger("bareasgi_graphql_next").addHandler(logging.NullHandler())
