@@ -2,22 +2,31 @@
 
 from abc import ABCMeta, abstractmethod
 import asyncio
-import io
-import json
-import logging
-
 from cgi import parse_multipart
 from datetime import datetime
-from typing import List, Dict, Any, Optional, Mapping
+from functools import partial
+import io
+import logging
+from typing import (
+    Any,
+    AsyncIterable,
+    Callable,
+    Dict,
+    List,
+    Mapping,
+    Optional,
+    Tuple,
+    Union,
+    cast
+)
 from urllib.parse import parse_qs, urlencode
 
 import graphql
-import bareutils.header as header
-
 from graphql import ExecutionResult
+from graphql.error.graphql_error import GraphQLError
+from graphql.execution import MiddlewareManager
 from graphql.subscription.map_async_iterator import MapAsyncIterator
 from bareasgi import Application
-from bareutils import text_reader, text_writer, response_code
 from baretypes import (
     Header,
     HttpResponse,
@@ -28,6 +37,8 @@ from baretypes import (
     WebSocket,
     HttpMiddlewareCallback
 )
+from bareutils import text_reader, text_writer, response_code
+import bareutils.header as header
 
 from .template import make_template
 from .utils import (
@@ -39,90 +50,66 @@ from .utils import (
     ZeroEvent
 )
 
-logger = logging.getLogger(__name__)
+LOGGER = logging.getLogger(__name__)
 
 
-def _make_sse_message(val: Optional[ExecutionResult]) -> str:
-    if val is None:
-        return f'event: ping\ndata: {datetime.utcnow()}\n\n'
+def _encode_sse(
+        dumps: Callable[[Any], str],
+        execution_result: Optional[ExecutionResult]
+) -> bytes:
+    if execution_result is None:
+        payload = f'event: ping\ndata: {datetime.utcnow()}\n\n'
+    else:
+        response = {
+            'data': execution_result.data,
+            'errors': [
+                error.formatted
+                for error in execution_result.errors
+            ] if execution_result.errors else None
+        }
 
-    response = {
-        'data': val.data,
-        'errors': val.errors
-    }
+        payload = f'event: message\ndata: {dumps(response)}\n\n'
 
-    return f'event: message\ndata: {json.dumps(response)}\n\n'
+    return payload.encode('utf-8')
 
 
-def _make_json_message(val: Optional[ExecutionResult]) -> str:
-    if val is None:
-        return '\n'
+def _encode_json(
+        dumps: Callable[[Any], str],
+        execution_result: Optional[ExecutionResult]
+) -> bytes:
+    if execution_result is None:
+        return b'\n'
 
-    return json.dumps({
-        'data': val.data,
-        'errors': val.errors
+    payload = dumps({
+        'data': execution_result.data,
+        'errors': [
+            error.formatted
+            for error in execution_result.errors
+        ] if execution_result.errors else None
     }) + '\n'
 
+    return payload.encode('utf-8')
+
 
 class GraphQLControllerBase(metaclass=ABCMeta):
     """GraphQL Controller Base"""
 
     def __init__(
             self,
-            path_prefix: str = '',
-            middleware=None,
-            ping_interval: float = 10
+            path_prefix: str,
+            middleware: Optional[Union[Tuple, List, MiddlewareManager]],
+            ping_interval: float,
+            loads: Callable[[str], Any],
+            dumps: Callable[[Any], str]
     ) -> None:
         self.path_prefix = path_prefix
         self.middleware = middleware
         self.ping_interval = ping_interval
+        self.loads = loads
+        self.dumps = dumps
         self.cancellation_event = asyncio.Event()
         self.subscription_count = ZeroEvent()
 
-    @abstractmethod
-    async def subscribe(
-            self,
-            query: str,
-            variables: Optional[Dict[str, Any]],
-            operation_name: Optional[str],
-            scope: Scope,
-            info: Info
-    ) -> MapAsyncIterator:
-        """Execute a subscription.
-
-        Args:
-            query (str): The subscription query.
-            variables (Optional[Dict[str, Any]]): Optional variables.
-            operation_name (Optional[str]): An optional operation name.
-            scope (Scope): The ASGI scope.
-            info (Info): The application info.
-
-        Returns:
-            MapAsyncIterator: An asynchronous iterator of the results.
-        """
-
-    @abstractmethod
-    async def query(
-            self,
-            query: str,
-            variables: Optional[Dict[str, Any]],
-            operation_name: Optional[str],
-            scope: Scope,
-            info: Info
-    ) -> ExecutionResult:
-        """Execute a query
-
-        Args:
-            query (str): The subscription query.
-            variables (Optional[Dict[str, Any]]): Optional variables.
-            operation_name (Optional[str]): An optional operation name.
-            scope (Scope): The ASGI scope.
-            info (Info): The application info.
-
-        Returns:
-            ExecutionResult: The query results.
-        """
-
     def add_routes(
             self,
             app: Application,
@@ -154,12 +141,12 @@ class GraphQLControllerBase(metaclass=ABCMeta):
         app.http_router.add(
             {'GET'},
             path_prefix + '/subscriptions',
-            wrap_middleware(rest_middleware, self.handle_sse_get)
+            wrap_middleware(rest_middleware, self.handle_subscription_get)
         )
         app.http_router.add(
             {'POST', 'OPTIONS'},
             path_prefix + '/subscriptions',
-            wrap_middleware(rest_middleware, self.handle_sse_post)
+            wrap_middleware(rest_middleware, self.handle_subscription_post)
         )
 
         # Add the subscription route
@@ -234,34 +221,6 @@ class GraphQLControllerBase(metaclass=ABCMeta):
             web_socket (WebSocket): The web socket to interact with
         """
 
-    @classmethod
-    async def _get_query_document(
-            cls,
-            headers: List[Header],
-            content: Content
-    ) -> Mapping[str, Any]:
-        content_type, parameters = header.content_type(headers)
-
-        if content_type == b'application/graphql':
-            return {'query': await text_reader(content)}
-        elif content_type in (b'application/json', b'text/plain'):
-            return json.loads(await text_reader(content))
-        elif content_type == b'application/x-www-form-urlencoded':
-            body = parse_qs(await text_reader(content))
-            return {name: value[0] for name, value in body.items()}
-        elif content_type == b'multipart/form-data':
-            return {
-                name: value[0]
-                for name, value in parse_multipart(
-                    io.StringIO(await text_reader(content)),
-                    {key.decode('utf-8'): val for key,
-                     val in parameters.items()}
-                ).items()
-            }
-        else:
-            raise RuntimeError('Content type not supported')
-
-    # pylint: disable=unused-argument
     async def handle_graphql(
             self,
             scope: Scope,
@@ -290,52 +249,27 @@ class GraphQLControllerBase(metaclass=ABCMeta):
 
             query_document = graphql.parse(query)
 
-            if has_subscription(query_document):
-                method = header.find(b'allow', scope['headers'], b'GET')
-                if method == b'GET':
-                    # Handle a subscription by returning 201 (Created) with
-                    # the url location of the subscription.
-                    scheme = scope['scheme']
-                    host = header.find(
-                        b'host', scope['headers'], b'localhost').decode()
-                    path = self.path_prefix + '/subscriptions'
-                    query_string = urlencode(
-                        {
-                            name.encode('utf-8'): json.dumps(value).encode('utf-8')
-                            for name, value in body.items()
-                        }
-                    )
-                    location = f'{scheme}://{host}{path}'
-                    location += f'?{query_string}'
-                    headers = [
-                        (b'access-control-expose-headers', b'location'),
-                        (b'location', location.encode('ascii'))
-                    ]
-                    return response_code.CREATED, headers
-                else:
-                    return await self._handle_sse(scope, info, body)
-            else:
-                # Handle a query
-                result = await self.query(
+            if not has_subscription(query_document):
+                return await self._handle_query_or_mutation(
+                    scope,
+                    info,
                     query,
                     variables,
-                    operation_name,
-                    scope,
-                    info
+                    operation_name
                 )
 
-                response: Dict[str, Any] = {'data': result.data}
-                if result.errors:
-                    response['errors'] = [
-                        error.formatted for error in result.errors]
+            # The subscription method is determined by the `allow` header.
+            allow = header.find(b'allow', scope['headers'], b'GET')
+            if allow == b'GET':
+                return self._handle_subscription_redirect(scope, body)
 
-                text = json.dumps(response)
-                headers = [
-                    (b'content-type', b'application/json'),
-                    (b'content-length', str(len(text)).encode())
-                ]
-
-                return 200, headers, text_writer(text)
+            return await self._handle_streaming_subscription(
+                scope,
+                info,
+                query,
+                variables,
+                operation_name
+            )
 
         # pylint: disable=bare-except
         except:
@@ -346,14 +280,14 @@ class GraphQLControllerBase(metaclass=ABCMeta):
             ]
             return response_code.INTERNAL_SERVER_ERROR, headers, text_writer(text)
 
-    async def handle_sse_get(
+    async def handle_subscription_get(
             self,
             scope: Scope,
             info: Info,
             matches: RouteMatches,
             content: Content
     ) -> HttpResponse:
-        """Handle a server sent event style direct subscription
+        """Handle a streaming subscription
 
         Args:
             scope (Scope): The ASGI scope
@@ -365,26 +299,39 @@ class GraphQLControllerBase(metaclass=ABCMeta):
             HttpResponse: The streaming response
         """
 
-        logger.debug(
-            'SSE received GET subscription request: http_version=%s',
+        LOGGER.debug(
+            "Received GET streaming subscription request: http_version='%s'.",
             scope['http_version']
         )
 
         body = {
-            name.decode('utf-8'): json.loads(value[0].decode('utf-8'))
-            for name, value in parse_qs(scope['query_string']).items()
+            name.decode('utf-8'): self.loads(value[0].decode('utf-8'))
+            for name, value in cast(
+                Dict[bytes, List[bytes]],
+                parse_qs(scope['query_string'])
+            ).items()
         }
 
-        return await self._handle_sse(scope, info, body)
+        query: str = body['query']
+        variables: Optional[Dict[str, Any]] = body.get('variables')
+        operation_name: Optional[str] = body.get('operationName')
 
-    async def handle_sse_post(
+        return await self._handle_streaming_subscription(
+            scope,
+            info,
+            query,
+            variables,
+            operation_name
+        )
+
+    async def handle_subscription_post(
             self,
             scope: Scope,
             info: Info,
             matches: RouteMatches,
             content: Content
     ) -> HttpResponse:
-        """Handle a server sent event style direct subscription
+        """Handle a streaming subscription
 
         Args:
             scope (Scope): The ASGI scope
@@ -396,39 +343,158 @@ class GraphQLControllerBase(metaclass=ABCMeta):
             HttpResponse: A stream response
         """
 
-        logger.debug(
-            'SSE received POST subscription request: http_version=%s',
+        LOGGER.debug(
+            "Received POST streaming subscription request: http_version='%s'.",
             scope['http_version']
         )
 
-        content = await text_reader(content)
-        body = json.loads(content)
+        text = await text_reader(content)
+        body = self.loads(text)
 
-        return await self._handle_sse(scope, info, body)
+        query: str = body['query']
+        variables: Optional[Dict[str, Any]] = body.get('variables')
+        operation_name: Optional[str] = body.get('operationName')
 
-    async def _handle_sse(
+        return await self._handle_streaming_subscription(
+            scope,
+            info,
+            query,
+            variables,
+            operation_name
+        )
+
+    async def _get_query_document(
+            self,
+            headers: List[Header],
+            content: Content
+    ) -> Mapping[str, Any]:
+        content_type = header.content_type(headers)
+        if content_type is None:
+            raise ValueError('Content type not specified')
+        media_type, parameters = content_type
+
+        if media_type == b'application/graphql':
+            return {'query': await text_reader(content)}
+        elif media_type in (b'application/json', b'text/plain'):
+            return self.loads(await text_reader(content))
+        elif media_type == b'application/x-www-form-urlencoded':
+            body = parse_qs(await text_reader(content))
+            return {name: value[0] for name, value in body.items()}
+        elif media_type == b'multipart/form-data':
+            if parameters is None:
+                raise ValueError(
+                    'Missing content type parameters for multipart/form-data'
+                )
+            return {
+                name: value[0]
+                for name, value in parse_multipart(
+                    io.StringIO(await text_reader(content)),
+                    {key.decode('utf-8'): val for key,
+                     val in parameters.items()}
+                ).items()
+            }
+        else:
+            raise RuntimeError(
+                f"Unsupported content type: {media_type.decode('ascii')}"
+            )
+
+    async def _handle_query_or_mutation(
             self,
             scope: Scope,
             info: Info,
-            body: Mapping[str, Any]
+            query: str,
+            variables: Optional[Dict[str, Any]],
+            operation_name: Optional[str]
     ) -> HttpResponse:
-        """Handle a server sent event style direct subscription"""
+        LOGGER.debug("Processing a query or mutation.")
 
-        accept = header.find(b'accept', scope['headers'], b'text/event-stream')
-        content_type = b'application/stream+json' if accept == b'application/json' else accept
-
-        result = await self.subscribe(
-            body['query'],
-            body.get('variables'),
-            body.get('operationName'),
+        result = await self.query(
+            query,
+            variables,
+            operation_name,
             scope,
             info
         )
 
-        # Make an async iterator for the subscription results.
+        response: Dict[str, Any] = {'data': result.data}
+        if result.errors:
+            response['errors'] = [
+                error.formatted for error in result.errors]
 
-        async def send_events(zero_event: ZeroEvent):
-            logger.debug('Started SSE subscription')
+        text = self.dumps(response)
+        headers = [
+            (b'content-type', b'application/json'),
+            (b'content-length', str(len(text)).encode())
+        ]
+
+        return 200, headers, text_writer(text)
+
+    def _handle_subscription_redirect(
+            self,
+            scope: Scope,
+            body: Mapping[str, Any]
+    ) -> HttpResponse:
+        # Handle a subscription by returning 201 (Created) with
+        # the url location of the subscription.
+        LOGGER.debug("Redirecting subscription request.")
+        scheme = scope['scheme']
+        host = cast(
+            bytes,
+            header.find(  # type: ignore
+                b'host',
+                scope['headers'],
+                b'localhost'
+            )
+        ).decode()
+        path = self.path_prefix + '/subscriptions'
+        query_string = urlencode(
+            {
+                name.encode('utf-8'): self.dumps(value).encode('utf-8')
+                for name, value in body.items()
+            }
+        )
+        location = f'{scheme}://{host}{path}?{query_string}'.encode('ascii')
+        headers = [
+            (b'access-control-expose-headers', b'location'),
+            (b'location', location)
+        ]
+        return response_code.CREATED, headers
+
+    async def _handle_streaming_subscription(
+            self,
+            scope: Scope,
+            info: Info,
+            query: str,
+            variables: Optional[Dict[str, Any]],
+            operation_name: Optional[str]
+    ) -> HttpResponse:
+
+        # If unspecified default to server sent events as they have better support.
+        accept = cast(
+            bytes,
+            header.find(b'accept', scope['headers'], b'text/event-stream')
+        )
+        content_type = (
+            b'application/stream+json'
+            if accept == b'application/json'
+            else accept
+        )
+
+        result = await self.subscribe(
+            query,
+            variables,
+            operation_name,
+            scope,
+            info
+        )
+
+        is_sse = content_type == b'text/event-stream'
+        encode = partial(_encode_sse if is_sse else _encode_json, self.dumps)
+        nudge = b':\n\n' if is_sse else b'\n'
+
+        # Make an async iterator for the subscription results.
+        async def send_events(zero_event: ZeroEvent) -> AsyncIterable[bytes]:
+            LOGGER.debug('Streaming subscription started.')
 
             try:
                 zero_event.increment()
@@ -438,26 +504,78 @@ class GraphQLControllerBase(metaclass=ABCMeta):
                         self.cancellation_event,
                         timeout=self.ping_interval
                 ):
-                    if content_type == b'text/event-stream':
-                        yield _make_sse_message(val).encode('utf-8')
-                        # Give the ASGI server a nudge.
-                        yield b':\n\n'
-                    else:
-                        yield _make_json_message(val).encode('utf-8')
-                        yield b'\n'
+                    yield encode(val)
+                    yield nudge  # Give the ASGI server a nudge.
 
             except asyncio.CancelledError:
-                logger.debug("Cancelled SSE subscription")
+                LOGGER.debug("Streaming subscription cancelled.")
+            except Exception as error:  # pylint: disable=broad-except
+                LOGGER.exception("Streaming subscription failed.")
+                # If the error is not caught the client fetch will fail, however
+                # the status code and headers have already been sent. So rather
+                # than let the fetch fail we send a GraphQL response with no
+                # data and the error and close gracefully.
+                if not isinstance(error, GraphQLError):
+                    error = GraphQLError(
+                        'Execution error',
+                        original_error=error
+                    )
+                val = ExecutionResult(None, [error])
+                yield encode(val)
+                yield nudge  # Give the ASGI server a nudge.
             finally:
                 zero_event.decrement()
 
-            logger.debug('Stopped SSE subscription')
+            LOGGER.debug("Streaming subscription stopped.")
 
         headers = [
             (b'cache-control', b'no-cache'),
-            # (b'content-type', b'text/event-stream'),
             (b'content-type', content_type),
             (b'connection', b'keep-alive')
         ]
 
         return response_code.OK, headers, send_events(self.subscription_count)
+
+    @abstractmethod
+    async def subscribe(
+            self,
+            query: str,
+            variables: Optional[Dict[str, Any]],
+            operation_name: Optional[str],
+            scope: Scope,
+            info: Info
+    ) -> MapAsyncIterator:
+        """Execute a subscription.
+
+        Args:
+            query (str): The subscription query.
+            variables (Optional[Dict[str, Any]]): Optional variables.
+            operation_name (Optional[str]): An optional operation name.
+            scope (Scope): The ASGI scope.
+            info (Info): The application info.
+
+        Returns:
+            MapAsyncIterator: An asynchronous iterator of the results.
+        """
+
+    @abstractmethod
+    async def query(
+            self,
+            query: str,
+            variables: Optional[Dict[str, Any]],
+            operation_name: Optional[str],
+            scope: Scope,
+            info: Info
+    ) -> ExecutionResult:
+        """Execute a query
+
+        Args:
+            query (str): The subscription query.
+            variables (Optional[Dict[str, Any]]): Optional variables.
+            operation_name (Optional[str]): An optional operation name.
+            scope (Scope): The ASGI scope.
+            info (Info): The application info.
+
+        Returns:
+            ExecutionResult: The query results.
+        """
