@@ -5,12 +5,13 @@ import graphene
 
 from .websocket_instance import GrapheneWebSocketHandlerInstance
 
+
 class GrapheneWebSocketHandler:
     """Graphene WebSocket handler"""
 
     def __init__(self, schema: graphene.Schema):
         """Graphene WebSocket handler
-        
+
         Args:
             schema (graphene.Schema): The schema
         """
@@ -23,5 +24,10 @@ class GrapheneWebSocketHandler:
             matches: RouteMatches,
             web_socket: WebSocket
     ) -> None:
-        instance = GrapheneWebSocketHandlerInstance(self.schema, web_socket, info)
+        instance = GrapheneWebSocketHandlerInstance(
+            self.schema,
+            web_socket,
+            scope,
+            info
+        )
         await instance.start(scope['subprotocols'])
